@@ -27,6 +27,10 @@ LOG = logging.getLogger(__name__)
 GENERATOR_OPTS = [
     cfg.StrOpt('output-file',
                help='Path of the file to write to. Defaults to stdout.'),
+    cfg.BoolOpt('exclude-deprecated',
+                default=False,
+                help='If True, exclude deprecated entries from the generated '
+                     'output.'),
 ]
 
 RULE_OPTS = [
@@ -177,9 +181,10 @@ def _format_rule_default_yaml(default, include_help=True, comment_rule=True,
         op = ""
         if hasattr(default, 'operations'):
             for operation in default.operations:
-                op += ('# %(method)s  %(path)s\n' %
-                       {'method': operation['method'],
-                        'path': operation['path']})
+                if operation['method'] and operation['path']:
+                    op += ('# %(method)s  %(path)s\n' %
+                           {'method': operation['method'],
+                            'path': operation['path']})
         intended_scope = ""
         if getattr(default, 'scope_types', None) is not None:
             intended_scope = (
@@ -199,29 +204,48 @@ def _format_rule_default_yaml(default, include_help=True, comment_rule=True,
             '# DEPRECATED\n# "%(name)s" has been deprecated since '
             '%(since)s.\n%(reason)s\n%(text)s'
         ) % {'name': default.name,
-             'check_str': default.check_str,
              'since': default.deprecated_since,
              'reason': _format_help_text(default.deprecated_reason),
              'text': text}
     elif add_deprecated_rules and default.deprecated_rule:
+        deprecated_reason = (
+            default.deprecated_rule.deprecated_reason or
+            default.deprecated_reason
+        )
+        deprecated_since = (
+            default.deprecated_rule.deprecated_since or
+            default.deprecated_since
+        )
+
         # This issues a deprecation warning but aliases the old policy name
         # with the new policy name for compatibility.
         deprecated_text = (
             '"%(old_name)s":"%(old_check_str)s" has been deprecated '
             'since %(since)s in favor of "%(name)s":"%(check_str)s".'
-        ) % {'old_name': default.deprecated_rule.name,
-             'old_check_str': default.deprecated_rule.check_str,
-             'since': default.deprecated_since,
-             'name': default.name,
-             'check_str': default.check_str,
-             }
-        text = ('%(text)s# DEPRECATED\n%(deprecated_text)s\n%(reason)s\n' %
-                {'text': text,
-                 'reason': _format_help_text(default.deprecated_reason),
-                 'deprecated_text': _format_help_text(deprecated_text)})
+        ) % {
+            'old_name': default.deprecated_rule.name,
+            'old_check_str': default.deprecated_rule.check_str,
+            'since': deprecated_since,
+            'name': default.name,
+            'check_str': default.check_str,
+        }
+        text = '%(text)s# DEPRECATED\n%(deprecated_text)s\n%(reason)s\n' % {
+            'text': text,
+            'reason': _format_help_text(deprecated_reason),
+            'deprecated_text': _format_help_text(deprecated_text)
+        }
 
         if default.name != default.deprecated_rule.name:
-            text += ('"%(old_name)s": "rule:%(name)s"\n' %
+            text += ('# WARNING: A rule name change has been identified.\n'
+                     '#          This may be an artifact of new rules being\n'
+                     '#          included which require legacy fallback\n'
+                     '#          rules to ensure proper policy behavior.\n'
+                     '#          Alternatively, this may just be an alias.\n'
+                     '#          Please evaluate on a case by case basis\n'
+                     '#          keeping in mind the format for aliased\n'
+                     '#          rules is:\n'
+                     '#          "old_rule_name": "new_rule_name".\n')
+            text += ('# "%(old_name)s": "rule:%(name)s"\n' %
                      {'old_name': default.deprecated_rule.name,
                       'name': default.name})
         text += '\n'
@@ -241,7 +265,7 @@ def _format_rule_default_json(default):
 
 
 def _sort_and_format_by_section(policies, output_format='yaml',
-                                include_help=True):
+                                include_help=True, exclude_deprecated=False):
     """Generate a list of policy section texts
 
     The text for a section will be created and returned one at a time. The
@@ -253,20 +277,24 @@ def _sort_and_format_by_section(policies, output_format='yaml',
     :param policies: A dict of {section1: [rule_default_1, rule_default_2],
                                 section2: [rule_default_3]}
     :param output_format: The format of the file to output to.
+    :param exclude_deprecated: If to exclude deprecated policy rule entries,
+                               defaults to False.
     """
     for section in sorted(policies.keys()):
         rule_defaults = policies[section]
         for rule_default in rule_defaults:
             if output_format == 'yaml':
-                yield _format_rule_default_yaml(rule_default,
-                                                include_help=include_help)
+                yield _format_rule_default_yaml(
+                    rule_default,
+                    include_help=include_help,
+                    add_deprecated_rules=not exclude_deprecated)
             elif output_format == 'json':
                 LOG.warning(policy.WARN_JSON)
                 yield _format_rule_default_json(rule_default)
 
 
 def _generate_sample(namespaces, output_file=None, output_format='yaml',
-                     include_help=True):
+                     include_help=True, exclude_deprecated=False):
     """Generate a sample policy file.
 
     List all of the policies available via the namespace specified in the
@@ -280,6 +308,8 @@ def _generate_sample(namespaces, output_file=None, output_format='yaml',
     :param include_help: True, generates a sample-policy file with help text
                          along with rules in which everything is commented out.
                          False, generates a sample-policy file with only rules.
+    :param exclude_deprecated: If to exclude deprecated policy rule entries,
+                               defaults to False.
     """
     policies = get_policies_dict(namespaces)
 
@@ -287,8 +317,10 @@ def _generate_sample(namespaces, output_file=None, output_format='yaml',
                    else sys.stdout)
 
     sections_text = []
-    for section in _sort_and_format_by_section(policies, output_format,
-                                               include_help=include_help):
+    for section in _sort_and_format_by_section(
+            policies, output_format,
+            include_help=include_help,
+            exclude_deprecated=exclude_deprecated):
         sections_text.append(section)
 
     if output_format == 'yaml':
@@ -304,7 +336,7 @@ def _generate_sample(namespaces, output_file=None, output_format='yaml',
         output_file.close()
 
 
-def _generate_policy(namespace, output_file=None):
+def _generate_policy(namespace, output_file=None, exclude_deprecated=False):
     """Generate a policy file showing what will be used.
 
     This takes all registered policies and merges them with what's defined in
@@ -312,6 +344,8 @@ def _generate_policy(namespace, output_file=None):
     that will be honored by policy checks.
 
     :param output_file: The path of a file to output to. stdout used if None.
+    :param exclude_deprecated: If to exclude deprecated policy rule entries,
+                               defaults to False.
     """
     enforcer = _get_enforcer(namespace)
     # Ensure that files have been parsed
@@ -327,7 +361,9 @@ def _generate_policy(namespace, output_file=None):
     output_file = (open(output_file, 'w') if output_file
                    else sys.stdout)
 
-    for section in _sort_and_format_by_section(policies, include_help=False):
+    for section in _sort_and_format_by_section(
+            policies, include_help=False,
+            exclude_deprecated=exclude_deprecated):
         output_file.write(section)
 
     if output_file != sys.stdout:
@@ -428,7 +464,8 @@ def _convert_policy_json_to_yaml(namespace, policy_file, output_file=None):
                 continue
             file_rule_check_str = file_policies.pop(default_rule.name)
             # Some rules might be still RuleDefault object so let's prepare
-            # empty 'operations' list for those.
+            # empty 'operations' list and rule name as description for
+            # those.
             operations = [{
                 'method': '',
                 'path': ''
@@ -442,7 +479,7 @@ def _convert_policy_json_to_yaml(namespace, policy_file, output_file=None):
             file_rule = policy.DocumentedRuleDefault(
                 default_rule.name,
                 file_rule_check_str,
-                default_rule.description,
+                default_rule.description or default_rule.name,
                 operations,
                 default_rule.deprecated_rule,
                 default_rule.deprecated_for_removal,
@@ -508,7 +545,9 @@ def generate_sample(args=None, conf=None):
     conf.register_opts(GENERATOR_OPTS + RULE_OPTS)
     conf(args)
     _check_for_namespace_opt(conf)
-    _generate_sample(conf.namespace, conf.output_file, conf.format)
+    _generate_sample(conf.namespace, output_file=conf.output_file,
+                     output_format=conf.format,
+                     exclude_deprecated=conf.exclude_deprecated)
 
 
 def generate_policy(args=None):
@@ -518,7 +557,8 @@ def generate_policy(args=None):
     conf.register_opts(GENERATOR_OPTS + ENFORCER_OPTS)
     conf(args)
     _check_for_namespace_opt(conf)
-    _generate_policy(conf.namespace, conf.output_file)
+    _generate_policy(conf.namespace, conf.output_file,
+                     conf.exclude_deprecated)
 
 
 def _upgrade_policies(policies, default_policies):
